unit BorderPage;

(*
   Developed by Mark Timmings 17/07/98
   Version 1.0

   The BorderPage is a pretty simple control based upon the
   cute border preview controls within Word97.

   First I'd like to say, the DotLine procedure is not
   particularily good, it just does it's job. Also all
   the Paint routines could probably do with tidying up.
   I needed to create this in a hurry so I haven't really
   written these as well as I would like to have (isn't
   that always the case) ? So if anyone feels like tidying
   things up please feel free.

   This is FREEWARE and as such you can do what you like
   with it, however if you carry out any major changes I'd
   very much like to have a copy. Other than that, I take
   no responsibility for any problems you have with the
   component.
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type

  TBorderLine = (blLeft, blTop, blRight, blBottom);
  TBorderLines = set of TBorderLine;

  TBorderPage = class(TGraphicControl)
  private
    { Private declarations }
    FBorderLines: TBorderLines;
    FOnChange: TNotifyEvent;
    FEnabledBorderLines: Boolean;
    FParagraphs: Boolean;
    FTextColor: TColor;
    FOnPaint: TNotifyEvent;
    FBorderLineColor: TColor;
    FBorderLineStyle: TPenStyle;
    FAllowChange: Boolean;
    FLeftMargin, FRightMargin: Integer;
    FTopMargin, FBottomMargin: Integer;
  protected
    { Protected declarations }
    procedure SetBorderLines(Value: TBorderLines);
    procedure SetEnabledBorderLines(Value: Boolean);
    procedure SetParagraphs(Value: Boolean);
    procedure SetTextColor(Value: TColor);
    procedure SetBorderLineColor(Value: TColor);
    procedure SetBorderLineStyle(Value: TPenStyle);
    procedure SetLeftMargin(Value: Integer);
    procedure SetTopMargin(Value: Integer);
    procedure SetRightMargin(Value: Integer);
    procedure SetBottomMargin(Value: Integer);

    procedure DotLine(Surface: TCanvas; StartPt: TPoint; l: Integer; Horz: Boolean);

    procedure PaintTop(Surface: TCanvas);
    procedure PaintBottom(Surface: TCanvas);
    procedure PaintLeft(Surface: TCanvas);
    procedure PaintRight(Surface: TCanvas);
    procedure PaintBorders(Surface: TCanvas);
    procedure PaintLines(Surface: TCanvas; EraseBk: Boolean);

    function  GetLeftRegion: TRect;
    function  GetTopRegion: TRect;
    function  GetRightRegion: TRect;
    function  GetBottomRegion: TRect;

    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property BorderLines: TBorderLines read FBorderLines write SetBorderLines;
    property Paragraphs: Boolean read FParagraphs write SetParagraphs default False;
    property EnabledBorderLines: Boolean read FEnabledBorderLines write SetEnabledBorderLines default True;
    property TextColor: TColor read FTextColor write SetTextColor default clSilver;
    property BorderLineColor: TColor read FBorderLineColor write SetBorderLineColor default clBlack;
    property BorderLineStyle: TPenStyle read FBorderLineStyle write SetBorderLineStyle default psSolid;
    property AllowChange: Boolean read FAllowChange write FAllowChange;
    property LeftMargin: Integer read FLeftMargin write SetLeftMargin default 0;
    property TopMargin: Integer read FTopMargin write SetTopMargin default 0;
    property RightMargin: Integer read FRightMargin write SetRightMargin default 0;
    property BottomMargin: Integer read FBottomMargin write SetBottomMargin default 0;

    property Align;
    property Color;
    property Visible;

    property OnBorderLineClick: TNotifyEvent read FOnChange write FOnChange;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;

    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
 end;

procedure Register;

implementation

const
   C_GAP = 20;          { gap from the edge of the paint area }
   C_INDENT = 10;       { idententation of the dotted horizontal lines }
   C_HORZ_WIDTH = 9;    { horizontal dotted line length }
   C_VERT_WIDTH = 7;    { vertical dotted line length }

constructor TBorderPage.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FBorderLines := [];
   FEnabledBorderLines := True;
   FParagraphs := False;
   Color := clWhite;
   FTextColor := clSilver;
   FBorderLineColor := clBlack;
   FBorderLineStyle := psSolid;
   FAllowChange := True;
   FLeftMargin := 0;
   FRightMargin := 0;
   FTopMargin := 0;
   FBottomMargin := 0;
   Width := 129;
   Height := 132;
end;

procedure TBorderPage.SetBorderLineStyle(Value: TPenStyle);
begin
   if Value <> FBorderLineStyle then
   begin
      FBorderLineStyle := Value;
      if FBorderLineStyle <> psClear then
         PaintBorders(Canvas)
      else
         Invalidate;
   end;
end;

procedure TBorderPage.SetEnabledBorderLines(Value: Boolean);
begin
   if Value <> FEnabledBorderLines then
   begin
      FEnabledBorderLines := Value;
      PaintBorders(Canvas);
   end;
end;

procedure TBorderPage.SetBorderLines(Value: TBorderLines);
begin
   if (Value <> FBorderLines) and FAllowChange  then
   begin
      FBorderLines := Value;
      PaintBorders(Canvas);
   end;
end;

procedure TBorderPage.SetBorderLineColor(Value: TColor);
begin
   if Value <> FBorderLineColor then
   begin
      FBorderLineColor := Value;
      PaintBorders(Canvas);
   end;
end;

procedure TBorderPage.SetLeftMargin(Value: Integer);
begin
   if Value <> FLeftMargin then
   begin
      FLeftMargin := Value;
      PaintLines(Canvas, True);
   end;
end;

procedure TBorderPage.SetTopMargin(Value: Integer);
begin
   if Value <> FTopMargin then
   begin
      FTopMargin := Value;
      PaintLines(Canvas, True);
   end;
end;

procedure TBorderPage.SetRightMargin(Value: Integer);
begin
   if Value <> FRightMargin then
   begin
      FRightMargin := Value;
      PaintLines(Canvas, True);
   end;
end;

procedure TBorderPage.SetBottomMargin(Value: Integer);
begin
   if Value <> FBottomMargin then
   begin
      FBottomMargin := Value;
      PaintLines(Canvas, True);
   end;
end;

procedure TBorderPage.SetTextColor(Value: TColor);
begin
   if Value <> FTextColor then
   begin
      FTextColor := Value;
      Invalidate;
   end;
end;

procedure TBorderPage.SetParagraphs(Value: Boolean);
begin
   if Value <> FParagraphs then
   begin
      FParagraphs := Value;
      Invalidate;
   end;
end;

procedure TBorderPage.DotLine(Surface: TCanvas; StartPt: TPoint; l: Integer; Horz: Boolean);
var
   pos: Integer;
begin
   if Horz then
   begin
      pos := StartPt.x;
      while pos < (StartPt.x + l) do
      begin
         Surface.Pixels[pos, StartPt.y] := Surface.Pen.Color;
         Inc(pos, 2);
      end;
   end
   else
   begin
      pos := StartPt.y;
      while pos < (StartPt.y + l) do
      begin
         Surface.Pixels[StartPt.x, pos] := Surface.Pen.Color;
         Inc(pos, 2);
      end;
   end;
end;

procedure TBorderPage.Paint;
var
   oldBColor, oldPColor: TColor;
begin
   oldBColor := Canvas.Brush.Color;
   Canvas.Brush.Color := Color;
   Canvas.FillRect(Rect(0, 0, Width, Height));

   oldPColor := Canvas.Pen.Color;
   Canvas.Pen.Color := FBorderLineColor;

   { Left Top }
   DotLine(Canvas, Point(C_INDENT - 1, C_GAP - 3), C_HORZ_WIDTH, True);
   DotLine(Canvas, Point(C_GAP - 3, C_INDENT - 1), C_VERT_WIDTH, False);

   { Right Top }
   DotLine(Canvas, Point((Width - C_HORZ_WIDTH - C_INDENT) + 3, C_GAP - 3), C_HORZ_WIDTH, True);
   DotLine(Canvas, Point((Width - C_HORZ_WIDTH - C_INDENT) + 3, C_INDENT - 1), C_VERT_WIDTH, False);

   { Bottom Left }
   DotLine(Canvas, Point(C_INDENT - 1, Height - (C_GAP - 3)), C_HORZ_WIDTH, True);
   DotLine(Canvas, Point(C_GAP - 3, Height - C_INDENT - C_VERT_WIDTH + 2), C_VERT_WIDTH, False);

   { Bottom Right }
   DotLine(Canvas, Point(Width - C_HORZ_WIDTH - C_INDENT + 3, Height - (C_GAP - 3)), C_HORZ_WIDTH, True);
   DotLine(Canvas, Point(Width - C_HORZ_WIDTH - C_INDENT + 3, Height - C_INDENT - C_VERT_WIDTH + 2), C_VERT_WIDTH, False);

   PaintLines(Canvas, False);
   PaintBorders(Canvas);

   if Assigned(FOnPaint) then
      FOnPaint(Self);

   Canvas.Brush.Color := oldBColor;
   Canvas.Pen.Color := oldPColor;
end;

procedure TBorderPage.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   pt: TPoint;
begin
   pt := Point(X, Y);
   { Check whether X, Y within any of the four sections }
   if PtInRect(GetLeftRegion, pt) then
   begin
      if not (blLeft in BorderLines) then
         BorderLines := BorderLines + [blLeft]
      else
         BorderLines := BorderLines - [blLeft];
      if Assigned(FOnChange) then
         FOnChange(Self);
   end
   else if PtInRect(GetTopRegion, pt) then
   begin
      if not (blTop in BorderLines) then
         BorderLines := BorderLines + [blTop]
      else
         BorderLines := BorderLines - [blTop];
      if Assigned(FOnChange) then
         FOnChange(Self);
   end
   else if PtInRect(GetRightRegion, pt) then
   begin
      if not (blRight in BorderLines) then
         BorderLines := BorderLines + [blRight]
      else
         BorderLines := BorderLines - [blRight];
      if Assigned(FOnChange) then
         FOnChange(Self);
   end
   else if PtInRect(GetBottomRegion, pt) then
   begin
      if not (blBottom in BorderLines) then
         BorderLines := BorderLines + [blBottom]
      else
         BorderLines := BorderLines - [blBottom];
      if Assigned(FOnChange) then
         FOnChange(Self);
   end;
   inherited MouseDown(Button, Shift, X, Y);
end;

function  TBorderPage.GetLeftRegion: TRect;
begin
   Result := Rect(0, C_GAP, C_GAP - 2, Height - C_GAP);
end;

function  TBorderPage.GetRightRegion: TRect;
begin
   Result := Rect((Width - C_GAP) + 3, C_GAP, Width, Height - C_GAP);
end;

function  TBorderPage.GetTopRegion: TRect;
begin
   Result := Rect(C_GAP - 1, 0, Width - C_GAP + 3, C_GAP);
end;

function  TBorderPage.GetBottomRegion: TRect;
begin
   Result := Rect(C_GAP - 2, Height - C_GAP, Width - C_GAP + 4, Height);
end;

procedure TBorderPage.PaintBorders(Surface: TCanvas);
begin
   PaintLeft(Surface);
   PaintTop(Surface);
   PaintRight(Surface);
   PaintBottom(Surface);
end;

procedure TBorderPage.PaintLines(Surface: TCanvas; EraseBk: Boolean);
var
   multiple: Integer;
   y: Integer;
   oldPWidth: Integer;
   oldPColor, oldBColor: TColor;
begin
   oldPWidth := Surface.Pen.Width;
   oldPColor := Surface.Pen.Color;
   oldBColor := Surface.Brush.Color;
   Surface.Pen.Width := 3;
   Surface.Pen.Color := FTextColor;
   Surface.Brush.Color := Color;

   if EraseBk then
      Canvas.FillRect(Rect(C_GAP - 1, C_GAP + 1, (Width - C_GAP) + 3, Height - C_GAP));

   multiple := 1;
   y := C_GAP + Surface.Pen.Width + FTopMargin;
   if FParagraphs then
   begin
      Surface.MoveTo(FLeftMargin + Round((Width - C_GAP) / 3), y);
      Surface.LineTo((Width - C_GAP - FRightMargin) + 1, y);
      Inc(y, Surface.Pen.Width * 2);
      multiple := 2;
   end;
   while y < (Height - C_GAP - (multiple * Surface.Pen.Width) - FBottomMargin - 1) do
   begin
      Surface.MoveTo(FLeftMargin + C_GAP, y);
      Surface.LineTo((Width - C_GAP - FRightMargin) + 1, y);
      Inc(y, Surface.Pen.Width * 2);
   end;
   if FParagraphs then
   begin
      Surface.MoveTo(FLeftMargin + C_GAP, y);
      Surface.LineTo(2 * (Round((Width - C_GAP) / 3)) - FRightMargin, y);
   end;
   Surface.Pen.Width := oldPWidth;
   Surface.Pen.Color := oldPColor;
   Surface.Brush.Color := oldBColor;
end;

procedure TBorderPage.PaintTop(Surface: TCanvas);
var
   r: TRect;
   oldPColor: TColor;
   oldPWidth: Integer;
   oldPStyle: TPenStyle;
begin
   oldPColor := Surface.Pen.Color;
   oldPWidth := Surface.Pen.Width;
   oldPStyle := Surface.Pen.Style;
   if (blTop in FBorderLines) and (FEnabledBorderLines) then
      Surface.Pen.Color := FBorderLineColor
   else
      Surface.Pen.Color := Color;

   Surface.Pen.Style := FBorderLineStyle;
   Surface.Pen.Width := 1;
   r := GetTopRegion;
   Surface.MoveTo(r.Left, r.Bottom);
   Surface.LineTo(r.Right, r.Bottom);

   Surface.Pen.Color := oldPColor;
   Surface.Pen.Width := oldPWidth;
   Surface.Pen.Style := oldPStyle;
end;

procedure TBorderPage.PaintBottom(Surface: TCanvas);
var
   r: TRect;
   oldPColor: TColor;
   oldPWidth: Integer;
   oldPStyle: TPenStyle;
begin
   oldPColor := Surface.Pen.Color;
   oldPWidth := Surface.Pen.Width;
   oldPStyle := Surface.Pen.Style;
   if (blBottom in FBorderLines) and (FEnabledBorderLines) then
      Surface.Pen.Color := FBorderLineColor
   else
      Surface.Pen.Color := Color;

   Surface.Pen.Style := FBorderLineStyle;
   Surface.Pen.Width := 1;
   r := GetBottomRegion;
   Surface.MoveTo(r.Left, r.Top);
   Surface.LineTo(r.Right, r.Top);

   Surface.Pen.Color := oldPColor;
   Surface.Pen.Width := oldPWidth;
   Surface.Pen.Style := oldPStyle;
end;

procedure TBorderPage.PaintLeft(Surface: TCanvas);
var
   r: TRect;
   oldPColor: TColor;
   oldPWidth: Integer;
   oldPStyle: TPenStyle;
begin
   oldPColor := Surface.Pen.Color;
   oldPWidth := Surface.Pen.Width;
   oldPStyle := Surface.Pen.Style;
   if (blLeft in FBorderLines) and (FEnabledBorderLines) then
      Surface.Pen.Color := FBorderLineColor
   else
      Surface.Pen.Color := Color;

   Surface.Pen.Style := FBorderLineStyle;
   Surface.Pen.Width := 1;
   r := GetLeftRegion;
   Surface.MoveTo(r.Right, r.Top);
   Surface.LineTo(r.Right, r.Bottom);

   Surface.Pen.Color := oldPColor;
   Surface.Pen.Width := oldPWidth;
   Surface.Pen.Style := oldPStyle;
end;

procedure TBorderPage.PaintRight(Surface: TCanvas);
var
   r: TRect;
   oldPColor: TColor;
   oldPWidth: Integer;
   oldPStyle: TPenStyle;
begin
   oldPColor := Surface.Pen.Color;
   oldPWidth := Surface.Pen.Width;
   oldPStyle := Surface.Pen.Style;
   if (blRight in FBorderLines) and (FEnabledBorderLines) then
      Surface.Pen.Color := FBorderLineColor
   else
      Surface.Pen.Color := Color;

   Surface.Pen.Style := FBorderLineStyle;
   Surface.Pen.Width := 1;
   r := GetRightRegion;
   Surface.MoveTo(r.Left, r.Top);
   Surface.LineTo(r.Left, r.Bottom);

   Surface.Pen.Color := oldPColor;
   Surface.Pen.Width := oldPWidth;
   Surface.Pen.Style := oldPStyle;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TBorderPage]);
end;

end.
